/*
 * TODO: get rid of malloc and fprintf
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>

#include "iff.h"

#define IFFREAD(name, addr, size) name->stream->stream_read(name->stream->stream, addr, size)
#define IFFSEEK(name, size, kind) name->stream->stream_seek(name->stream->stream, size, kind)
#define IFFTELL(name) name->stream->stream_tell(name->stream->stream)
#define IFFDONE(name) name->stream->stream_done(name->stream->stream)

static iff_uhalf bswaph(iff_uhalf h)
{
  iff_uhalf hb = (iff_uhalf) ((h & 0x00ff) << 8);
  iff_uhalf lb = (iff_uhalf) ((h >> 8) & 0x00ff);
  return (iff_uhalf) ((hb & 0xff00) + (lb & 0x00ff)); 
}

static iff_size bswaps(iff_size s)
{
  iff_uhalf hw = bswaph((iff_uhalf) (s & 0xffff));
  iff_uhalf lw = bswaph((iff_uhalf) (s >> 16));
  return (iff_size) ((hw << 16) + lw);
}

struct IFFTag * iff_init(struct IFFStreamTag * stream)
{
  iff_name name;
  iff_size size;
  struct IFFTag * form;

  form = malloc(IFFTAGSZ);
  _CHECK(!form, "malloc(%u) failed", IFFTAGSZ, 0);

  form->size = 0;
  form->name = MKNAME('f', 'r', 'e', 'e');
  form->stream = stream;
  form->parent = 0;

  name = iff_readname(form);
  size = iff_readsize(form);
  IFFCHK(name != FORM, form);

  form->name = name;
  form->size = size;
  return form;
}

struct IFFTag * iff_child(struct IFFTag * parent, iff_size size, iff_name name)
{
  struct IFFTag * form = malloc(sizeof(struct IFFTag));
  IFFCHK(!form, parent);
  form->name = name;
  form->size = size ;
  form->stream = parent->stream;
  form->parent = parent;
  return form;
}

iff_void iff_close(struct IFFTag * iff)
{
  if (iff->stream) IFFDONE(iff);
  while (iff)
  {
    struct IFFTag * tmp = iff->parent;
    free(iff);
    iff = tmp;
  }
}

iff_void iff_error(struct IFFTag * iff,
                   const char * cond, const char * file, int line)
{
  if (iff->stream) IFFDONE(iff);
  fprintf(stderr, "iff check failed in %s(%d): %s, stream %s\n",
          file, line, cond, iff->stream->stream_name);
  
  fprintf(stderr, "current chunk chain ");
  for (; iff; iff = iff->parent)
  {
    char name[5] = "none";
    *(iff_name *) &name = iff->name;
    name[4] = 0;
    fprintf(stderr, "%s ", name);
  }
  fprintf(stderr, "\n");
  fflush(stderr);
  exit(2);
}

iff_half iff_readhalf(struct IFFTag * iff)
{
  iff_half r;
  IFFREAD(iff, &r, 2);
  return r;
}

iff_size iff_readsize(struct IFFTag * iff)
{
  iff_size r;
  IFFREAD(iff, &r, 4);
  return bswaps(r);
}

iff_word iff_readword(struct IFFTag * iff)
{
  iff_word r;
  IFFREAD(iff, &r, 4);
  return r;
}

iff_char iff_readchar(struct IFFTag * iff)
{
  iff_char r;
  IFFREAD(iff, &r, 1);
  return r;
}

iff_size iff_readdata(struct IFFTag * iff, void * data, iff_size size)
{
  return IFFREAD(iff, data, size);
}

iff_void iff_align(struct IFFTag * iff)
{
  if (IFFTELL(iff) & 1)
  {
    IFFSEEK(iff, 1, 1);
    iff->size--;
  }
}

iff_void iff_skip(struct IFFTag * iff, iff_size size)
{
  IFFSEEK(iff, size, 1);
}

