#ifndef __iff_included
#define __iff_included

#include "config.h"

#include "cpppro.h"

typedef void iff_void;
typedef int iff_sword;
typedef short int iff_half;
typedef unsigned int iff_word;
typedef unsigned int iff_name;
typedef int iff_size;
typedef unsigned char iff_char;
typedef unsigned short int iff_uhalf;

#define IFFSTREAMTAGSZ sizeof(struct IFFStreamTag)
struct IFFStreamTag
{
  void * stream;
  const char * stream_name;
  iff_size (* stream_read)(void *, void *, iff_size);
  iff_size (* stream_seek)(void *, iff_size, int);
  iff_size (* stream_tell)(void *);
  iff_void (* stream_done)(void *);
};

#define IFFTAGSZ sizeof(struct IFFTag)
struct IFFTag
{
  iff_size size;
  iff_name name;
  struct IFFTag * parent;
  struct IFFStreamTag * stream;
};

#define MKNAME(a, b, c, d) (iff_name)((a)+((b)<<8)+((c)<<16)+((d)<<24))
#define FORM MKNAME('F', 'O', 'R', 'M')
#define IFFCHK(cond, iff) \
(void) ((cond) ? iff_error(iff, #cond, __FILE__, __LINE__), 0 : 0)

#define UCHUNK(name, iff)\
{ \
  char s[20]; \
  sprintf(s,  "unknown chunk %c%c%c%c", \
  name, \
  name >> 8, \
  name >> 16, \
  name >> 24); \
  iff_error(iff, s, __FILE__, __LINE__); \
}

extern struct IFFTag * iff_init(struct IFFStreamTag *);
extern struct IFFTag * iff_child(struct IFFTag *, iff_size, iff_name);

extern iff_void iff_close(struct IFFTag *);
extern iff_void iff_error(struct IFFTag *, const char *, const char *, int);
extern iff_half iff_readhalf(struct IFFTag *);
extern iff_size iff_readsize(struct IFFTag *);
extern iff_word iff_readword(struct IFFTag *);
extern iff_char iff_readchar(struct IFFTag *);
extern iff_void iff_align(struct IFFTag *);
extern iff_void iff_skip(struct IFFTag *, iff_size);
extern iff_size iff_readdata(struct IFFTag *, void *, iff_size);

#define iff_readname(a) (iff_name) (iff_readword(a))

#include "cppepi.h"

#endif /* __iff_included */
