#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iff.h"
#include "mtn.h"
#include "skl.h"

#define MOTN MKNAME('M', 'O', 'T', 'N')
#define PRLE MKNAME('P', 'R', 'L', 'E')
#define POSE MKNAME('P', 'O', 'S', 'E')

static struct MotnTag MotionRoot;
static struct MotnTag * Motion = &MotionRoot;

struct MotnTag * mtn_init(struct IFFStreamTag * stream)
{
  struct IFFTag * motn = iff_init(stream);
  iff_name name = iff_readname(motn);

  IFFCHK(name != MOTN, motn);

  motn->size -= 4;
  motn->name = name;

  while (motn->size)
  {
    iff_name name = iff_readname(motn);
    iff_size size = iff_readsize(motn);

    switch (name)
    {
      case MOTN:
      {
        IFFCHK(size != 83, motn);
        iff_readdata(motn, &Motion->motn, size);
        iff_align(motn);
        break;
      }

      case PRLE:
      {
        struct MotnTag * Motn;
        iff_word index = 0;
        iff_word deltas = Motion->motn.PoseCount;
        iff_word angles = Motion->motn.PoseCount * Motion->motn.LimbCount * 3;
        iff_word packed = size - deltas * MOTNDELTATAGSZ;

        iff_char * temp;
        iff_char * prle = malloc(packed);
        Motion->DeltaList = malloc(deltas * MOTNDELTATAGSZ);
        Motion->AngleList = malloc(angles);

        IFFCHK(!Motion->DeltaList, motn);
        IFFCHK(!Motion->AngleList, motn);

        iff_readdata(motn, Motion->DeltaList, deltas * MOTNDELTATAGSZ);
        iff_readdata(motn, prle, packed);

        temp = prle;
        while (packed--)
        {
          iff_char count = (iff_char) *prle++;
          if (count > 0x80)
          {
            iff_char angle = *prle++;
            count &= ~0x80;
            packed--;
            while (count--) Motion->AngleList[index++] = angle;
          } else
          {
            packed -= count;
            while (count--) Motion->AngleList[index++] = *prle++;
          }
        }
        iff_align(motn);

        free(temp);

        Motn = malloc(MOTNTAGSZ);
        IFFCHK(!Motn, motn);
        Motn->link = Motion;
        Motion = Motn;
      } break;

      case POSE:
      {
        struct MotnTag * Motn;
        
        int i, off;
        iff_char * data;
	iff_word delta = Motion->motn.LimbCount * 8 + MOTNDELTATAGSZ;
        iff_word deltas = Motion->motn.PoseCount;
        iff_word angles = Motion->motn.PoseCount * Motion->motn.LimbCount * 3;

        data = malloc(size);
        Motion->DeltaList = malloc(deltas * MOTNDELTATAGSZ);
        Motion->AngleList = malloc(angles);

        fprintf(stderr, 
                "%s motion is POSE, telescopic motion will be discarded\n",
                Motion->motn.name);

        IFFCHK(!data, motn);
        IFFCHK(!Motion->DeltaList, motn);
        IFFCHK(!Motion->AngleList, motn);

        iff_readdata(motn, data, size);

        for (i = 0, off = 0; i < Motion->motn.PoseCount; i++, off += delta)
        {
          int j;

          memcpy(Motion->DeltaList, data + off, MOTNDELTATAGSZ);

          for (j = 0; j < Motion->motn.LimbCount; j++)
          {
            int p_delta = j * Motion->motn.PoseCount + i;
            int a_delta = Motion->motn.PoseCount * Motion->motn.LimbCount;

            Motion->AngleList[p_delta + a_delta * 0] = data[off + j * 8 + 18];
            Motion->AngleList[p_delta + a_delta * 1] = data[off + j * 8 + 19];
            Motion->AngleList[p_delta + a_delta * 2] = data[off + j * 8 + 20];
          }
        }

        free(data);

        Motn = malloc(MOTNTAGSZ);
        IFFCHK(!Motn, motn);
        Motn->link = Motion;
        Motion = Motn;

        iff_align(motn);
      } break;

      default:
        UCHUNK(name, motn);
        break;
    }
    motn->size -= size + 8;
    IFFCHK(motn->size < 0, motn);
  }
  iff_close(motn);
  return Motion;
}

#ifdef _TEST
int main(void)
{
  mtn_init("man.mtn");
  return 0;
}
#endif
