#include "config.h"

#include <stdlib.h>

#include "iff.h"
#include "skl.h"

#define SKEL MKNAME('S', 'K', 'E', 'L')
#define VERS MKNAME('V', 'E', 'R', 'S')
#define LIMB MKNAME('L', 'I', 'M', 'B')
#define LOCK MKNAME('L', 'O', 'C', 'K')
#define SENS MKNAME('S', 'E', 'N', 'S')

struct SkelTag * skl_init(struct IFFStreamTag * stream)
{
  int GotLimbs = 0;
  struct IFFTag * skel = iff_init(stream);
  struct SkelTag * Skeleton = malloc(SKELTAGSZ);
  iff_name name = iff_readname(skel);

  IFFCHK(!Skeleton, skel);
  IFFCHK(name != SKEL, skel);
 
  skel->size -= 4;
  skel->name = name;

  while (skel->size)
  {
    iff_name name = iff_readname(skel);
    iff_size size = iff_readsize(skel);

    switch (name)
    {
      case VERS:
      {
        iff_half version = iff_readhalf(skel);
        IFFCHK(version != 2, skel);
      } break;

      case LIMB:
      {
        IFFCHK(GotLimbs == 1, skel);

        Skeleton->LimbCount = size / 39;
        Skeleton->LimbList = malloc(size);
        IFFCHK(!Skeleton->LimbList, skel);

        iff_readdata(skel, Skeleton->LimbList, size);
        iff_align(skel);
        GotLimbs = 1;
      } break;

      case LOCK:
      case SENS: iff_skip(skel, size); iff_align(skel); break;

      default:
        UCHUNK(name, skel);
        break;
    }
    skel->size -= size + 8;
    IFFCHK(skel->size < 0, skel);
  }
  IFFCHK(GotLimbs != 1, skel);
  iff_close(skel);
  return Skeleton;
}

#ifdef _TEST
int main(void)
{
  skl_init("man.skl");
  return 0;
}
#endif
