#include "config.h"

#include <stdlib.h>
#include <string.h>
#include "real.h"

static const uchar RealPalette[768];

static void hocus(struct RetrTag * retr)
{
  int i;
  #define _MAX(l, r) l = l > r ? l : r
  #define _MIN(l, r) l = l < r ? l : r

  long maxx = retr->real->VertexList[0].x;
  long maxy = retr->real->VertexList[0].y;
  long maxz = retr->real->VertexList[0].z;
  long minx = maxx;
  long miny = maxy;
  long minz = maxz;
  float xc;
  float yc;
  float zc;
  float rs;

  for (i = 0; i < retr->real->VertexCount; i++)
  {
    #define VTX(a) retr->real->VertexList[i].a
    _MAX(maxx, VTX(x));
    _MAX(maxy, VTX(y));
    _MAX(maxz, VTX(z));
    _MIN(minx, VTX(x));
    _MIN(miny, VTX(y));
    _MIN(minz, VTX(z));
    #undef VTX
  }
  
  xc = (float) (maxx + minx) / 2.f;
  yc = (float) (maxy + miny) / 2.f;
  zc = (float) (maxz + minz) / 2.f;

  rs =     (float) (maxx - minx);
  _MAX(rs, (float) (maxy - miny));
  _MAX(rs, (float) (maxz - minz));

  retr->rs = 1.f / rs;
  retr->xc = xc;
  retr->yc = yc;
  retr->zc = zc;

  #undef _MAX
  #undef _MIN
}

struct RetrTag * real_init(const char * name)
{
  int i;
  int id;
  struct RealTag * real = eor_init(stream_init(name));
  struct RetrTag * retr = malloc(RETRTAGSZ);

  _CHECK(!retr, "malloc(%u) failed", RETRTAGSZ, 0);

  retr->real     = real;
  retr->palette  = RealPalette;
  retr->MapCount = real->TrisFaceCount + real->QuadFaceCount;
  retr->MapList  = malloc(MAPTAGSZ * retr->MapCount);

  _CHECK(!retr->MapList, "malloc(%u) failed", MAPTAGSZ * retr->MapCount, 0);

  for (i = 0; i < retr->MapCount; i++)
  {
    retr->MapList[i].MapType = MAP_TYPE_NONE;
  }

  for (i = 0; i < real->TrisMapCount; i++)
  {
    id = real->TrisMapList[i].id;
    retr->MapList[id].MapType     = MAP_TYPE_TRIS;
    retr->MapList[id].Map.TrisMap = &real->TrisMapList[i];
  }

  for (i = 0; i < real->QuadMapCount; i++)
  {
    id = real->QuadMapList[i].id + real->TrisFaceCount;
    retr->MapList[id].MapType     = MAP_TYPE_QUAD;
    retr->MapList[id].Map.QuadMap = &real->QuadMapList[i];
  }

  retr->VertexList = malloc(sizeof(float) * real->VertexCount * 3);
  _CHECK(!retr->VertexList, "malloc(%u) failed",
         sizeof(float) * real->VertexCount * 3, 0);

  hocus(retr);
  return retr;
}

void real_done(struct RetrTag * retr)
{
  #define _FREE(p) if (p) free(p)
  _FREE(retr->MapList);
  _FREE(retr->VertexList);

  if (retr->real)
  {
    _FREE(retr->real->LineFaceList);
    _FREE(retr->real->TrisFaceList);
    _FREE(retr->real->QuadFaceList);
    _FREE(retr->real->TrisMapList);
    _FREE(retr->real->QuadMapList);
    _FREE(retr->real->VertexList);
  
    if (retr->real->TextureList)
    {
      int i;

      for (i = 0; i < retr->real->TextureCount; i++)
      {
        free(retr->real->TextureList[i].bitmap);
      }
      free(retr->real->TextureList);
    }
    free(retr->real);
  }
  free(retr);
  #undef _FREE
}

static const uchar RealPalette[768] =
{
  0x0c , 0x0c , 0x0c , 0x1c , 0x14 , 0x18 , 0x28 , 0x1c , 0x20 , 0x24 ,
  0x24 , 0x28 , 0x28 , 0x28 , 0x34 , 0x2c , 0x2c , 0x30 , 0x34 , 0x24 ,
  0x2c , 0x40 , 0x2c , 0x34 , 0x34 , 0x34 , 0x3c , 0x38 , 0x38 , 0x44 ,
  0x38 , 0x3c , 0x48 , 0x3c , 0x3c , 0x40 , 0x40 , 0x40 , 0x4c , 0x44 ,
  0x44 , 0x4c , 0x48 , 0x48 , 0x54 , 0x4c , 0x4c , 0x58 , 0x50 , 0x50 ,
  0x5c , 0x54 , 0x54 , 0x60 , 0x58 , 0x58 , 0x64 , 0x5c , 0x50 , 0x5c ,
  0x64 , 0x44 , 0x50 , 0x70 , 0x48 , 0x54 , 0x74 , 0x54 , 0x60 , 0x84 ,
  0x58 , 0x64 , 0x90 , 0x64 , 0x74 , 0xa0 , 0x64 , 0x78 , 0xb0 , 0x70 ,
  0x84 , 0x8c , 0x8c , 0x98 , 0x84 , 0x84 , 0x94 , 0x7c , 0x7c , 0x88 ,
  0x74 , 0x74 , 0x84 , 0x78 , 0x68 , 0x7c , 0x68 , 0x68 , 0x7c , 0x58 ,
  0x68 , 0x78 , 0x60 , 0x60 , 0x6c , 0x4c , 0x58 , 0x68 , 0x48 , 0x4c ,
  0x78 , 0x3c , 0x40 , 0x78 , 0x38 , 0x40 , 0x68 , 0x38 , 0x50 , 0x68 ,
  0x28 , 0x50 , 0x6c , 0x18 , 0x4c , 0x68 , 0x14 , 0x40 , 0x5c , 0x28 ,
  0x48 , 0x5c , 0x38 , 0x44 , 0x58 , 0x28 , 0x3c , 0x4c , 0x28 , 0x2c ,
  0x48 , 0x2c , 0x30 , 0x58 , 0x2c , 0x30 , 0x6c , 0x2c , 0x5c , 0x78 ,
  0x34 , 0x64 , 0x84 , 0x30 , 0x70 , 0x90 , 0x20 , 0x70 , 0x9c , 0x30 ,
  0x80 , 0xa8 , 0x3c , 0x90 , 0xb4 , 0x38 , 0x94 , 0xc4 , 0x2c , 0x8c ,
  0xc0 , 0x20 , 0x94 , 0xd0 , 0x30 , 0x9c , 0xd4 , 0x44 , 0xa0 , 0xd4 ,
  0x40 , 0xb0 , 0xe0 , 0x30 , 0xb0 , 0xf0 , 0x3c , 0xbc , 0xf4 , 0x30 ,
  0xbc , 0xfc , 0x38 , 0xcc , 0xfc , 0x54 , 0xd8 , 0xf4 , 0x54 , 0xe4 ,
  0xfc , 0x7c , 0xe0 , 0xfc , 0x78 , 0xfc , 0xfc , 0x8c , 0xfc , 0xfc ,
  0x94 , 0xc4 , 0xe8 , 0x80 , 0xb4 , 0xe8 , 0xa0 , 0xa4 , 0xf8 , 0x88 ,
  0x8c , 0xdc , 0x74 , 0x88 , 0xe0 , 0x70 , 0x80 , 0xc4 , 0x70 , 0x9c ,
  0xc4 , 0x84 , 0xa0 , 0xb4 , 0x94 , 0x94 , 0xa8 , 0x9c , 0x9c , 0xa8 ,
  0xa0 , 0xa0 , 0xb4 , 0xa8 , 0xa8 , 0xc0 , 0xb4 , 0xb4 , 0xcc , 0xbc ,
  0xbc , 0xd4 , 0xc8 , 0xc8 , 0xdc , 0xd4 , 0xd4 , 0xe4 , 0x60 , 0xb8 ,
  0xc8 , 0x58 , 0xbc , 0xec , 0x58 , 0x9c , 0xe0 , 0x54 , 0x94 , 0xc0 ,
  0x54 , 0x70 , 0xbc , 0x58 , 0x5c , 0xb0 , 0x58 , 0x6c , 0xa4 , 0x44 ,
  0x70 , 0x9c , 0x44 , 0x58 , 0x90 , 0x44 , 0x48 , 0x8c , 0x48 , 0x54 ,
  0xa0 , 0x54 , 0x58 , 0x94 , 0x58 , 0x70 , 0x88 , 0x78 , 0x78 , 0x94 ,
  0x70 , 0x7c , 0xa0 , 0x88 , 0x88 , 0xa8 , 0x4c , 0x50 , 0xc4 , 0x1c ,
  0x80 , 0xb4 , 0x18 , 0x5c , 0x84 , 0x10 , 0x34 , 0x48 , 0x0c , 0x28 ,
  0x38 , 0x1c , 0x20 , 0x40 , 0x18 , 0x1c , 0x28 , 0x4c , 0x34 , 0x3c ,
  0x5c , 0x3c , 0x48 , 0x24 , 0xa4 , 0xe8 , 0xdc , 0x60 , 0x48 , 0xe0 ,
  0x70 , 0x40 , 0xe8 , 0x84 , 0x34 , 0xf0 , 0x9c , 0x2c , 0xf8 , 0xbc ,
  0x20 , 0xfc , 0xe0 , 0x14 , 0xfc , 0xfc , 0x14 , 0xfc , 0xfc , 0x30 ,
  0xfc , 0xfc , 0xb0 , 0xfc , 0xfc , 0xfc , 0x2c , 0x24 , 0x54 , 0x44 ,
  0x38 , 0xac , 0x48 , 0x4c , 0xfc , 0x88 , 0x88 , 0xfc , 0xc4 , 0xc8 ,
  0xfc , 0xd4 , 0xfc , 0xfc , 0x0c , 0x0c , 0x0c , 0x10 , 0x10 , 0x10 ,
  0x18 , 0x18 , 0x18 , 0x20 , 0x20 , 0x20 , 0x28 , 0x28 , 0x28 , 0x30 ,
  0x30 , 0x30 , 0x38 , 0x38 , 0x38 , 0x40 , 0x40 , 0x40 , 0x48 , 0x48 ,
  0x48 , 0x50 , 0x50 , 0x50 , 0x58 , 0x58 , 0x58 , 0x60 , 0x60 , 0x60 ,
  0x6c , 0x6c , 0x6c , 0x74 , 0x74 , 0x74 , 0x7c , 0x7c , 0x7c , 0x84 ,
  0x84 , 0x84 , 0x8c , 0x8c , 0x8c , 0x94 , 0x94 , 0x94 , 0x9c , 0x9c ,
  0x9c , 0xa4 , 0xa4 , 0xa4 , 0xac , 0xac , 0xac , 0xb4 , 0xb4 , 0xb4 ,
  0xbc , 0xbc , 0xbc , 0xc4 , 0xc4 , 0xc4 , 0xcc , 0xcc , 0xcc , 0xd4 ,
  0xd4 , 0xd4 , 0xdc , 0xdc , 0xdc , 0xe4 , 0xe4 , 0xe4 , 0xec , 0xec ,
  0xec , 0xf8 , 0xf8 , 0xf8 , 0xfc , 0xfc , 0xfc , 0xfc , 0xfc , 0xfc ,
  0x30 , 0x24 , 0x1c , 0x38 , 0x28 , 0x20 , 0x44 , 0x34 , 0x28 , 0x4c ,
  0x38 , 0x2c , 0x58 , 0x40 , 0x34 , 0x60 , 0x48 , 0x38 , 0x6c , 0x54 ,
  0x40 , 0x74 , 0x58 , 0x44 , 0x80 , 0x60 , 0x4c , 0x88 , 0x68 , 0x54 ,
  0x94 , 0x70 , 0x5c , 0xa0 , 0x78 , 0x64 , 0xa8 , 0x80 , 0x68 , 0xb4 ,
  0x88 , 0x70 , 0xbc , 0x90 , 0x7c , 0xc8 , 0x98 , 0x84 , 0xd4 , 0xa0 ,
  0x8c , 0xd4 , 0xa4 , 0x90 , 0xd8 , 0xac , 0x94 , 0xdc , 0xb4 , 0x9c ,
  0xe0 , 0xbc , 0xa0 , 0xe4 , 0xc0 , 0xa4 , 0xe8 , 0xc8 , 0xac , 0xec ,
  0xd0 , 0xb0 , 0xf0 , 0xd4 , 0xb8 , 0xf0 , 0xdc , 0xbc , 0xf4 , 0xe0 ,
  0xc4 , 0xf8 , 0xe8 , 0xc8 , 0xfc , 0xec , 0xd0 , 0xfc , 0xf4 , 0xd4 ,
  0xfc , 0xf8 , 0xdc , 0xfc , 0xfc , 0xe4 , 0x14 , 0x30 , 0x14 , 0x1c ,
  0x40 , 0x1c , 0x24 , 0x54 , 0x2c , 0x30 , 0x68 , 0x3c , 0x3c , 0x7c ,
  0x4c , 0x4c , 0x90 , 0x60 , 0x5c , 0xa4 , 0x78 , 0x6c , 0xb8 , 0x90 ,
  0x80 , 0xcc , 0xa8 , 0x90 , 0xd4 , 0xb0 , 0xa0 , 0xdc , 0xbc , 0xb0 ,
  0xe4 , 0xc4 , 0xc0 , 0xec , 0xd0 , 0xd0 , 0xf4 , 0xdc , 0xe4 , 0xfc ,
  0xec , 0xf8 , 0xfc , 0xfc , 0x5c , 0x44 , 0x4c , 0x50 , 0x3c , 0x44 ,
  0x3c , 0x28 , 0x30 , 0x6c , 0x50 , 0x5c , 0x5c , 0x5c , 0x64 , 0x64 ,
  0x64 , 0x70 , 0x60 , 0x60 , 0x74 , 0x70 , 0x70 , 0x84 , 0x88 , 0x5c ,
  0x6c , 0xa4 , 0xa4 , 0xb4 , 0xb0 , 0xa8 , 0xb4 , 0xc0 , 0xc0 , 0xe4 ,
  0xd0 , 0xd0 , 0xf4 , 0xe4 , 0xe4 , 0xf4 , 0xec , 0xec , 0xfc , 0xfc ,
  0xfc , 0xfc , 0x4c , 0x18 , 0x0c , 0x60 , 0x1c , 0x0c , 0x78 , 0x24 ,
  0x14 , 0x90 , 0x30 , 0x20 , 0xa8 , 0x3c , 0x2c , 0xc0 , 0x48 , 0x3c ,
  0xd8 , 0x58 , 0x50 , 0xdc , 0x60 , 0x64 , 0xe4 , 0x70 , 0x78 , 0xe8 ,
  0x80 , 0x8c , 0xec , 0x94 , 0xa0 , 0xf0 , 0xa4 , 0xb4 , 0xf8 , 0xb8 ,
  0xc8 , 0xfc , 0xcc , 0xd8 , 0xfc , 0xe0 , 0xec , 0xfc , 0xf4 , 0xfc ,
  0x18 , 0x34 , 0x48 , 0x1c , 0x40 , 0x54 , 0x24 , 0x4c , 0x60 , 0x30 ,
  0x58 , 0x6c , 0x38 , 0x68 , 0x78 , 0x48 , 0x74 , 0x88 , 0x54 , 0x84 ,
  0x94 , 0x60 , 0x94 , 0xa0 , 0x74 , 0xa4 , 0xac , 0x84 , 0xb4 , 0xbc ,
  0x94 , 0xc0 , 0xc8 , 0xa8 , 0xd0 , 0xd4 , 0xbc , 0xe0 , 0xe0 , 0xd4 ,
  0xec , 0xec , 0xec , 0xfc , 0xfc , 0xfc , 0x0c , 0xfc
};

