#include "config.h"

#include <stdlib.h>
#include "skin.h"

struct SntrTag * skin_init(const char * name)
{
  struct SkinTag * skin;
  struct SntrTag * sntr;
  
  sntr = malloc(SNTRTAGSZ);
  _CHECK(!sntr, "malloc(%u) failed", SNTRTAGSZ, 0);

  skin = skn_init(stream_init(name));

  sntr->skin = skin;
  sntr->VertLimbList = 0;
  return sntr;
}

void skin_done(struct SntrTag * sntr)
{
  #define _FREE(p) if (p) free(p)
  _FREE(sntr->VertLimbList);
  if (sntr->skin)
  {
    _FREE(sntr->skin->SkinList);
    free(sntr->skin);
  }
  free(sntr);   
  #undef _FREE
}

static int skin_comp(const void * l, const void * r)
{
  const int * li = l;
  const int * ri = r;
  return *li > *ri ? 1 : -1;
}

void skin_process(struct SntrTag * sntr,
                  struct RetrTag * retr)
{
  int i;
  int * VertLimbList;

  #define n retr->real->VertexCount

  _CHECK(retr->real->VertexCount > sntr->skin->SkinCount,
         "%d %d", retr->real->VertexCount, sntr->skin->SkinCount);

  if (n != sntr->skin->SkinCount)
  {
    fprintf(stderr,
            NEARSTR " warning vertex count not equal to skin count %d, %d\n",
            NEARPOS, n, sntr->skin->SkinCount);
  }

  VertLimbList = malloc((n + 1) * sizeof(int) * 2);
  _CHECK(!VertLimbList, "malloc(%u) failed",
         (n + 1) * sizeof(int) * 2, 0);

  sntr->VertLimbList = VertLimbList;
           
  VertLimbList[n * 2 + 0] = -1;
  VertLimbList[n * 2 + 1] = -1;

  for (i = 0; i < n; i++)
  {
    VertLimbList[i * 2 + 0] = sntr->skin->SkinList[i];
    VertLimbList[i * 2 + 1] = i;
  }

  qsort(VertLimbList, n, sizeof(int) * 2, skin_comp);

  #if 0
  printf("\nSorted:\n");
  for (i = 0; i < sntr->skin->SkinCount; i++)
  {
    printf("VertLimbList[%d] Limb %d Vert %d\n",
    i,
    VertLimbList[i * 2 + 0],
    VertLimbList[i * 2 + 1]);
  }
  fflush(stdout);
  #endif
}
